﻿using FellowOakDicom.Network;
using FellowOakDicom.Network.Client;
using FellowOakDicom.Network.Client.EventArguments;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AssociationDicom
{
    class Program
    {
        static async Task Main(string[] args)
        {
            try
            {
                //replace these with your settings
                //Here, I am using Dr.Dave Harvey's public server
                //please be careful not to send any confidential info as all traffic is logged
                var dicomRemoteHost = "www.dicomserver.co.uk";
                var dicomRemoteHostPort = 104;
                var useTls = false;
                var ourDotNetTestClientDicomAeTitle = "TESTAET";
                var remoteDicomHostAeTitle = "DAVEHARVEYS";

                //создаем клиент эхо-проверки DICOM с обработчиками
                var client = DicomClientFactory.Create(dicomRemoteHost, dicomRemoteHostPort, useTls, ourDotNetTestClientDicomAeTitle, remoteDicomHostAeTitle);
                var dicomCEchoRequest = new DicomCEchoRequest();

                dicomCEchoRequest.OnResponseReceived += OnEchoResponseReceivedFromRemoteHost;

                //добавляем обработчики событий для общей информации о подключении ассоциации
                client.AssociationAccepted += ClientOnAssociationAccepted;                                
                client.AssociationRejected += ClientOnAssociationRejected;
                client.AssociationReleased += ClientOnAssociationReleased;

                await client.AddRequestAsync(dicomCEchoRequest).ConfigureAwait(true);

                //отправляем запрос на проверку на удаленный сервер DICOM
                await client.SendAsync().ConfigureAwait(true);
                
                Console.ReadLine();

            }
            catch (Exception e)
            {
                //In real life, do something about this exception
                LogToDebugConsole($"Error occured during DICOM association request -> {e.StackTrace}");
            }
        }

        
        private static void OnEchoResponseReceivedFromRemoteHost(DicomCEchoRequest request, DicomCEchoResponse response)
        {
            LogToDebugConsole($"DICOM Echo Verification request was received by remote host");
            LogToDebugConsole($"Response was received from remote host...");
            LogToDebugConsole($"Verification response status returned was:{response.Status.ToString()}");
            Console.ReadLine();
        }

        private static void ClientOnAssociationReleased(object sender, EventArgs e)
        {
            LogToDebugConsole("Assoacition was released");
            Console.ReadLine();
        }

        private static void ClientOnAssociationRejected(object sender, AssociationRejectedEventArgs e)
        {
            LogToDebugConsole($"Association was rejected. Rejected Reason:{e.Reason}");
            Console.ReadLine();
        }

        private static void ClientOnAssociationAccepted(object sender, AssociationAcceptedEventArgs e)
        {
            var association = e.Association;
            LogToDebugConsole($"Association was accepted by remote host: {association.RemoteHost} running on port: {association.RemotePort}");

            foreach (var presentationContext in association.PresentationContexts)
            {
                if (presentationContext.Result == DicomPresentationContextResult.Accept)
                {
                    LogToDebugConsole($"\t {presentationContext.AbstractSyntax} was accepted");
                    LogToDebugConsole($"\t Negotiation result was: {presentationContext.GetResultDescription()}");
                    LogToDebugConsole($"\t Abstract syntax accepted: {presentationContext.AbstractSyntax}");
                    LogToDebugConsole($"\t Transfer syntax accepted: {presentationContext.AcceptedTransferSyntax.ToString()}");
                }
                else
                {
                    LogToDebugConsole($"\t Presentation context with proposed abstract syntax of '{presentationContext.AbstractSyntax}' was not accepted");
                    LogToDebugConsole($"\t Reject reason was {presentationContext.GetResultDescription()}");
                }
            }
            Console.ReadLine();
        }

        private static void LogToDebugConsole(string informationToLog)
        {
            Console.WriteLine(informationToLog);
        }
    }
}
